import java.awt.*;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;
import java.text.DateFormat;
import java.util.Locale;

import javax.print.PrintService;
import javax.print.attribute.AttributeSet;
import javax.print.attribute.standard.JobName;
import javax.swing.*;

import jpdf.attribute.*;
import jpdf.gui.dialog.PrintDialog;
import jpdf.gui.dialog.PrintDialogPreferences;
import jpdf.gui.dialog.SetupContentDialog;
import jpdf.gui.dialog.factory.SetupContentPrintDialogFactory;
import jpdf.gui.preview2.PageLayout;
import jpdf.gui.preview2.PreviewPrintableDialog;
import jpdf.manage.PrintManager;
import jpdf.manage.PrintParameterHolder;
import jpdf.manage.ResourceAdministrator;
import jpdf.printer.TreeComponentPrinter;


/**
 * Class SetupPreviewPrintFrame implements a frame which displays a tree containing 
 * the Java files in the Sample Code directory, and contains buttons which implement 
 * Page Setup, Print Preview, and Print functionality.  These buttons allow the user 
 * to preview and print the displayed tree.  
 *
 * <P>This class provides an example of how to use the JPDF to implement printing 
 * capabilities.
 */
public class SetupPreviewPrintFrame
  extends JFrame
{
  //-----------------------------------------------------------------------
  // Inner Classes.
  //-----------------------------------------------------------------------

  /**
   * Window Close Listener.  The Application is terminated when the frame is 
   * closed.
   */
  private static class WindowCloseListener 
    extends WindowAdapter
  {
    public void windowClosing(WindowEvent theEvent)
    {
      // Shutdown the JPDF Environment.
      PrintManager.shutdownEnvironment();
      
      // Exit the application.
      System.exit(0); 
    }
  }

  /**
   * The Page Setup button displays the Page Setup Dialog which allows the user to 
   * specify the Print Attributes which control the page that will be used to print 
   * the tree.
   */  
  private class PageSetupButton
    extends JButton
    implements ActionListener
  {
    private PageSetupButton() 
    {
      super("Page Setup");
      
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent theEvent)
    {
      SetupContentDialog theDialog = null;

      // Create and display the Page Setup Dialog.
      setWaitCursor(); 
      theDialog = SetupContentDialog.createTreeComponentContentDialog(
                    SetupPreviewPrintFrame.this, PrintManager.getDefaultPrintAttributes());
      setDefaultCursor();
      theDialog.show(); 
    }  
  }

  /**
   * The Preview button displays the Print Preview Dialog which allows the 
   * user to preview the tree.
   */
  private class PreviewButton
    extends JButton
    implements ActionListener
  {
    private PreviewButton() 
    {
      super("Preview");
      
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      PreviewPrintableDialog thePreviewDialog = null;
      TreeComponentPrinter   thePrinter = null;
      
      // Create and display the Preview Dialog.
      // NOTE: The Dialog Factory creates the Page Setup and Print dialogs that can 
      //       be displayed from within the Preview Dialog.
      setWaitCursor(); 
      thePrinter = new TreeComponentPrinter(SAMPLE_TREE, TITLE, true);
      thePreviewDialog = new PreviewPrintableDialog(SetupPreviewPrintFrame.this, 
                                                    thePrinter);
      thePreviewDialog.setDialogFactory(new SetupContentPrintDialogFactory()); 
      setDefaultCursor(); 
      thePreviewDialog.show(); 
    }
  }

  /**
   * Print Button.  Displays the Print Dialog which allows the user to specify the 
   * Print Attributes which control the Print Job that will be used to print the 
   * tree, and to submit this Print Job.
   */
  private class PrintButton
    extends JButton
    implements ActionListener
  {
    private PrintButton() 
    {
      super("Print");
      
      addActionListener(this);
    }

    public void actionPerformed(ActionEvent theEvent) 
    {
      PrintService thePrintService = null;
      PrintDialog  theDialog = null;

      // Create and display the Print Dialog.
      setWaitCursor(); 
      theDialog = new PrintDialog(SetupPreviewPrintFrame.this, 
                                  PrintManager.getDefaultPrintAttributes(), true);
      setDefaultCursor(); 
      theDialog.show(); 
      
      // Get the Selected Print Service.
      // NOTE: Null is returned if the user did not press the Print Button.
      thePrintService = theDialog.getSelectedPrintService();
      if (thePrintService != null) 
      {
        PrintManager.printPrintable(new TreeComponentPrinter(SAMPLE_TREE, TITLE, true), 
                                    theDialog.getPrintInfoHolder());
      }
    }
  }

  
  //-----------------------------------------------------------------------
  // Class variables/constants.
  //-----------------------------------------------------------------------
  private static final JTree  SAMPLE_TREE;
  private static final String TITLE = "Sample Code Java Files";
  
  
  //-----------------------------------------------------------------------
  // Class Initialization.
  //-----------------------------------------------------------------------
  static
  {
    JTree theTree = null;

    // Set the Look-And-Feel.
    try
    {
      UIManager.setLookAndFeel(UIManager.getSystemLookAndFeelClassName());
    }
    catch (Exception theEx) {}  
    
    // Create the tree.
    try
    {
      theTree = new JavaFileTree("..");
    }
    catch (Exception theEx) 
    {
      System.out.println("***ERROR: " + theEx);
      System.exit(-99);
    }  
    SAMPLE_TREE = theTree;
  }
  
  
  //-----------------------------------------------------------------------
  // Constructors.
  //-----------------------------------------------------------------------
  
  private SetupPreviewPrintFrame() 
  {
    super(TITLE);

    // Initialize the frame contents.
    initializeContents();
  }

    
  //-----------------------------------------------------------------------
  // Public methods.
  //-----------------------------------------------------------------------

  /**
   * Application entry point.
   */
  public static void main(String theParams[]) 
  {
    // Initialize the JPDF Environment.
    initializeJpdfEnvironment();
    
    // Display the frame.  
    (new SetupPreviewPrintFrame()).setVisible(true);
  }      
   
  /**
   * Sets the cursor to the default cursor.
   */  
  public void setDefaultCursor()
  {
    setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
    getGlassPane().setVisible(false);
  }

  /**
   * Sets the cursor to the wait cursor.
   */  
  public void setWaitCursor()
  {
    getGlassPane().setVisible(true);
    setCursor(Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR));
  }

   
  //-----------------------------------------------------------------------
  // Private methods.
  //-----------------------------------------------------------------------

  /**
   * Creates the Button Panel.
   */
  private JPanel createButtonPanel()
  {
    JPanel theButtonPanel = new JPanel();
    
    theButtonPanel.add(new PageSetupButton());
    theButtonPanel.add(new PreviewButton());
    theButtonPanel.add(new PrintButton());
    
    return theButtonPanel;
  }

  /**
   * Initializes the contents of this frame.
   */
  private void initializeContents()
  {
    Container theContentPane = getContentPane();
    
    // Configure the frame.
    setSize(600, 400);
    setLocationRelativeTo(null);

    // Add the contents.
    theContentPane.add((new JScrollPane(SAMPLE_TREE)), BorderLayout.CENTER);
    theContentPane.add(createButtonPanel(), BorderLayout.SOUTH);
    
    // Add listeners.
    addWindowListener(new WindowCloseListener());
  }
  
  /**
   * Initializes the JPDF environment.
   */
  private static void initializeJpdfEnvironment() 
  {
    AttributeSet  theAttributeSet = null; 
    String        theDateAndTimeMarker = null;
    String        thePageAndCountMarker = null;
    Font          thePageNoteFont = UIManager.getFont("Label.font");
    
    // Initialize the JPDF Environment.
    try
    {
      PrintManager.initializeEnvironment();
    }
    catch (Throwable theEx)
    {
      theEx.printStackTrace();
      System.exit(99); 
    }  
     
    // Preload the list of media supported by the Default Print Service on a background thread.  
    // This speeds up the initial display of the Page Setup dialog.
    PrintManager.preloadPrintServiceMediaOnThread();
    
    // Specify the Date Format used in Page Headers/Footers.
    ResourceAdministrator.setTimeStyle(DateFormat.SHORT);
    
    // Print Service selection will be done using icons.
    PrintDialogPreferences.setToUsePrintServiceSelectionIcons(true);
    
    // Do not include the Banner Page Controller in the Print Dialogs.
    PrintDialogPreferences.setToIncludeBannerPageController(false);
    
    // Configure the Preview Dialog preferences.
    PreviewPrintableDialog.setToUsePreviousSizeLocation(true);
    PreviewPrintableDialog.setInitialPageLayout(PageLayout.SINGLE_COL);
    
    // Get the Default Attribute Set.
    theAttributeSet = PrintManager.getDefaultPrintAttributes(); 
    
    // Add the initial attributes we want to use for printing.
    // NOTE: Special markers are used in the Page Header/Footer.  These are substituted by actual values
    //       when the material is printed.
    theDateAndTimeMarker = PrintParameterHolder.DATE_MARKER + " " + PrintParameterHolder.TIME_MARKER;
    thePageAndCountMarker = "Page " + PrintParameterHolder.PAGE_NUMBER_MARKER + " of " + 
                            PrintParameterHolder.PAGE_COUNT_MARKER;
    theAttributeSet.add(new JobName("Preview Tree Example", null));
    theAttributeSet.add(new PageHeadNotes("", PrintParameterHolder.TITLE_MARKER, "", Color.BLUE, 
                                          thePageNoteFont.deriveFont(24f)));
    theAttributeSet.add(new PageFootNotes(theDateAndTimeMarker, "", thePageAndCountMarker, Color.BLUE, 
                                          thePageNoteFont.deriveFont(12f)));
    theAttributeSet.add(new AlignmentFactorHorizontal(0.5f));
    theAttributeSet.add(new OffsetTop(0.1f, true));
    theAttributeSet.add(PageFrame.TRUE);
    theAttributeSet.add(PageContentFrame.TRUE);
    theAttributeSet.add(UseBackgroundColor.TRUE);
    theAttributeSet.add(BackgroundColorOnPage.FALSE);
    theAttributeSet.add(HighlightSelections.TRUE);    
  }
}